REM Pebble language and compiler for X86 DOS
REM Geek Basic 2022 http://www.basicgames.xyz
REM Send questions and suggestions to ptrworkmails@gmail.com

CONST lines = 4500
CONST includes = 100
CONST nest = 100

DIM code$(lines)
DIM incs$(includes)
DIM ifstack(nest)

LET crlf$ = CHR$(13) + CHR$(10)

PRINT "Pebble Compiler"

GOSUB load
GOSUB compile

END

load:

	PRINT "Loading..."

	LET iolib$ = ""

	OPEN COMMAND$ FOR INPUT AS #1

	FOR count = 0 TO lines

		IF EOF(1) THEN

			EXIT FOR

		END IF

		IF count = lines THEN

			PRINT "Line count exceeded!"
			SLEEP
			END

		END IF

		LINE INPUT #1, code$(count)

		GOSUB removetabs
		LET code$(count) = LTRIM$(RTRIM$(code$(count)))
		GOSUB programcmd
		GOSUB includecmd
		GOSUB usecmd

	NEXT count

	LET count = count + 1

	FOR ucount = 0 TO include - 1

		LET code$(count) = incs$(ucount)
		GOSUB includecmd

	NEXT ucount

	CLOSE #1

RETURN

compile:

	PRINT "Compiling..."

	LET inlineasm = 0

	OPEN title$ FOR OUTPUT AS #1

	FOR count = 0 TO lines - 1

		IF LEN(code$(count)) AND LEFT$(code$(count), 1) <> ";" THEN

			PRINT #1, ";" + STR$(count) + " " + code$(count)

		END IF

		IF inlineasm AND UCASE$(code$(count)) <> "ENDASM" THEN

			PRINT #1, code$(count)
			LET cmd$ = ""

		ELSEIF LEFT$(code$(count), 1) = ";" OR LEFT$(code$(count), 1) = "[" OR LEFT$(code$(count), 1) = "@" THEN

			LET cmd$ = LEFT$(code$(count), 1)
			LET parameters$ = code$(count)

		ELSE

			IF UCASE$(LEFT$(code$(count),4))<>"STR$" AND UCASE$(LEFT$(code$(count),4))<>"FIND" AND UCASE$(LEFT$(code$(count),7))<>"COMPARE" THEN

				GOSUB convertsinglequotes

			END IF

			LET space = INSTR(code$(count), " ")

			IF space THEN

				LET cmd$ = LEFT$(code$(count), space - 1)
				LET parameters$ = MID$(code$(count), space + 1, LEN(code$(count)) - space)

			ELSE

				LET cmd$ = code$(count)
				LET parameters$ = ""

			END IF

		END IF

		SELECT CASE UCASE$(cmd$)

			CASE "DATA"

				GOSUB defsection
				LET dataflag = 1

			CASE "INT", "INT@", "STR$"

				LET varname$ = LEFT$(parameters$, INSTR(parameters$, "[") - 1)
				LET parameters$ = LEFT$(parameters$, LEN(parameters$) - 1)
				LET parameters$ = RIGHT$(parameters$, LEN(parameters$) - LEN(varname$) - 1)
				LET vardata$ = parameters$

				IF UCASE$(cmd$)="INT" THEN

					PRINT #3, varname$ + " dw " + vardata$

				ELSEIF UCASE$(cmd$)="INT@" THEN

	 				PRINT #3, varname$ + " dw " + vardata$ + " dup(0)"

				ELSEIF UCASE$(cmd$)="STR$" THEN

					PRINT #3, varname$ + " db " + vardata$ + ",0"
					PRINT #3, "rb 1023"

				END IF

			CASE "BEGIN"

				IF dataflag = 0 THEN

					GOSUB defsection

				END IF

				OPEN "header.dat" FOR INPUT AS #2

				WHILE NOT EOF(2)

					LINE INPUT #2, headerdata$
					PRINT #1, headerdata$

				WEND

				CLOSE #2

			CASE "[", "@"

				LET expression$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(parameters$, "=") + 1, LEN(parameters$))))

				IF INSTR(expression$, "(") OR INSTR(expression$, "&") OR INSTR(expression$, "|") OR INSTR(parameters$, "@") THEN

					GOSUB letcmd

				ELSEIF INSTR(expression$, "=") OR INSTR(expression$, "<") OR INSTR(expression$, ">") THEN

					GOSUB letcmd

				ELSEIF INSTR(expression$, CHR$(128)) OR INSTR(expression$, CHR$(129)) OR INSTR(expression$, CHR$(130)) THEN

					GOSUB letcmd

				ELSEIF INSTR(expression$, "/") OR INSTR(expression$, "*") OR INSTR(expression$, "+") OR INSTR(expression$, "-") THEN

					LET operators = 0

					FOR check = 1 to LEN(expression$)

						SELECT CASE MID$(expression$,check,1)

							CASE "/", "*", "+", "-"

								LET operators = operators + 1

						END SELECT

					NEXT check

					IF operators > 1 THEN

						GOSUB letcmd

					ELSE

						GOSUB simplelet

					END IF

				ELSE

					GOSUB simplelet

				END IF


			CASE "IF"


				IF INSTR(parameters$, "(") OR INSTR(parameters$, "&") OR INSTR(parameters$, "|") OR INSTR(parameters$, "@")THEN

					GOSUB ifcmd

				ELSEIF INSTR(parameters$, "/") OR INSTR(parameters$, "*") OR INSTR(parameters$, "+") OR INSTR(parameters$, "-") THEN

					GOSUB ifcmd

				ELSE

					GOSUB simpleif

				END IF

			CASE "ENDIF"

				LET ifpointer = ifpointer - 1
				PRINT #1, ".endif" + LTRIM$(STR$(ifstack(ifpointer))) + ":"

			CASE "CARRY"

				PRINT #1, "mov " + parameters$ + ",dx"

			CASE "0"

				PRINT #1, "mov " + parameters$ + ",0"

			CASE "+1"

				PRINT #1, "add " + parameters$ + ",1"

			CASE "-1"

				PRINT #1, "sub " + parameters$ + ",1"

			CASE "GOTO"

				IF LEFT$(parameters$,1) <> "@" THEN


					PRINT #1, "jmp ." + parameters$

				ELSE

					PRINT #1, "jmp " + parameters$

				END IF

			CASE "LABEL"

				IF LEFT$(parameters$,1) <> "@" THEN

					PRINT #1, "." + parameters$ + ":"

				ELSE

					PRINT #1, "@@:"

				END IF

			CASE "SUB"

				PRINT #1, parameters$ + ":"

			CASE "CALL"

				PRINT #1, "call " + parameters$

			CASE "RET"

				PRINT #1, "ret"

			CASE "ECHO"

				GOSUB echocmd

			CASE "INPUT"

				GOSUB inputcmd

			CASE "CRLF"

				LET newlineflag = 1
				LET echostringflag = 1
				PRINT #1, "call newline"

			CASE "BELL"

				LET bellflag = 1
				LET echostringflag = 1
				PRINT #1, "call ring"

			CASE "CHR"

				LET chrflag = 1
				LET echostringflag = 1
				GOSUB chrcmd

			CASE "CURSOR"

				LET cursorflag = 1
				GOSUB cursorcmd

			CASE "PAUSE"

				LET pauseflag = 1
				PRINT #1, "call pauseprogram"

			CASE "SCANKEY"

				LET scankeyflag = 1
				PRINT #1, "call scankey"
				PRINT #1, "mov " + parameters$ + ",ax"

			CASE "ONKEY"

				LET onkeyflag = 1
				PRINT #1, "call onkey"
				PRINT #1, "je ." + parameters$

			CASE "COPY"

				LET copyflag = 1
				GOSUB copycmd

			CASE "CONCAT"

				LET concatflag = 1
				GOSUB concatcmd

			CASE "COMPARE"

				LET compareflag = 1
				GOSUB comparecmd

			CASE "LENGTH"

				LET lengthflag = 1
				GOSUB lengthcmd

			CASE "FIND"

				LET findflag = 1
				GOSUB findcmd

			CASE "TRIM"

				LET lengthflag = 1
				LET trimflag = 1
				GOSUB trimcmd

			CASE "CUT"

				LET cutflag = 1
				GOSUB cutcmd

			CASE "INTSTR"

				LET intstrflag = 1
				GOSUB intstrcmd

			CASE "STRINT"

				LET strintflag = 1
				GOSUB strintcmd

			CASE "UPPER"

				LET upperflag = 1
				GOSUB uppercmd

			CASE "LOWER"

				LET lowerflag = 1
				GOSUB lowercmd

			CASE "CLS"

				LET clsflag = 1
				GOSUB clscmd

			CASE "WAIT"

				LET waitflag = 1
				GOSUB waitcmd

			CASE "KILL"

				PRINT #1, "int 32"

			CASE "BEGINASM"

				LET inlineasm = 1

			CASE "ENDASM"

				LET inlineasm = 0

			CASE ";"

				PRINT #1, parameters$

			CASE ELSE

				IF cmd$ <> "" AND UCASE$(cmd$) <> "END" THEN

					PRINT "Invalid command on line #" + STR$(count + 1) + ": " + code$(count)

				END IF

		END SELECT

	NEXT count

	CLOSE #3

	OPEN "io.lib" FOR INPUT AS #2

	WHILE NOT EOF(2)

		LINE INPUT #2, libdata$

		LET count = lines - 1
		LET code$(count) = libdata$
		GOSUB removetabs
		LET libdata$ = LTRIM$(RTRIM$(code$(count)))

		SELECT CASE LCASE$(libdata$)

			CASE "inputvalue:", "getvalue:", "inputstring:"

				IF inputflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strinput:"

				IF strinputflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strecho:"

				IF strechoflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strcharecho:"

				IF strechoflag <> 1 AND strinputflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strcopy:"

				IF copyflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strconcat:"

				IF concatflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strcmp:"

				IF compareflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strlen:"

				IF lengthflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strfind:"

				IF findflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strtrim:","strltrim:","strrtrim:"

				IF trimflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strcut:"

				IF cutflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strintstr:"

				IF intstrflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strstrint:"

				IF strintflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strupper:"

				IF upperflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "strlower:"

				IF lowerflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "scankey:"

				IF scankeyflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "onkey:"

				IF onkeyflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "pauseprogram:"

				IF pauseflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "echostring:"

				IF echostringflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "echovalue:"

				IF echovalueflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "newline:"

				IF newlineflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "movecursor:"

				IF cursorflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "echochar:"

				IF chrflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "ring:"

				IF bellflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "clearscreen:", "resetcolors:"

				IF clsflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "waitfor:"

				IF waitflag <> 1 THEN

					LET nolib = 1

				ELSE

					PRINT #1, libdata$

				END IF

			CASE "crlf db 13,10,36"

				IF newlineflag = 1 THEN

					PRINT #1, libdata$

				END IF

			CASE "bell db 7,36"

				IF bellflag = 1 THEN

					PRINT #1, libdata$

				END IF

			CASE "inputbuffer db 8,0", "digits rb 12"

				IF inputflag = 1 THEN

					PRINT #1, libdata$

				END IF

			CASE "ret"

				IF nolib <> 1 THEN

					PRINT #1, libdata$

				ELSE

					LET nolib = 0

				END IF

			CASE ELSE

				IF nolib = 0 THEN

					PRINT #1, libdata$

				END IF

		END SELECT

	WEND

	CLOSE #2

	PRINT "Copying variable definitions..."

	OPEN "defs.dat" FOR INPUT AS #3

	WHILE NOT EOF(3)

		LINE INPUT #3, vardef$
		PRINT #1, vardef$

	WEND

	CLOSE #3

	SHELL "del defs.dat"

	CLOSE #1

	PRINT "Compilation is finished."

RETURN

defsection:

	OPEN "defs.dat" FOR OUTPUT AS #3
	PRINT #3, ""
	CLOSE #3

	OPEN "defs.dat" FOR APPEND AS #3

RETURN

removetabs:

	LET notabs$ = ""

	FOR tabscan = 1 TO LEN(code$(count))

		LET char$ = MID$(code$(count), tabscan, 1)

		IF char$ <> CHR$(9) THEN

			LET notabs$ = notabs$ + char$

		END IF

	NEXT tabscan

	LET code$(count) = notabs$

RETURN

convertsinglequotes:

	LET nosinglequotes$ = ""

	FOR singlequotescan = 1 TO LEN(code$(count))

		LET char$ = MID$(code$(count), singlequotescan, 1)

		IF char$ <> CHR$(39) THEN

			LET nosinglequotes$ = nosinglequotes$ + char$

		ELSE

			LET nosinglequotes$ = nosinglequotes$ + "',39,'"

		END IF

	NEXT singlequotescan

	LET code$(count) = nosinglequotes$	

RETURN

programcmd:

	IF UCASE$(LEFT$(code$(count), 7)) = "PROGRAM" THEN

		LET parameters$ = MID$(code$(count), 9, LEN(code$(count)) - 8)

		IF parameters$ <> "" THEN

			LET title$ = parameters$ + ".asm"

		END IF

		LET code$(count) = ""

	END IF

RETURN

includecmd:

	IF UCASE$(LEFT$(code$(count), 7)) = "INCLUDE" THEN

		LET includefile$ = MID$(code$(count), 9, LEN(code$(count)) - 8)

		PRINT "Including: " + includefile$ + "..."

		OPEN includefile$ FOR INPUT AS #2

		FOR icount = count TO lines

			IF EOF(2) THEN EXIT FOR

			IF icount = lines THEN

				PRINT "Line count exceeded in " + includefile$ + "!"
				SLEEP
				END

			END IF

			LINE INPUT #2, code$(icount)
			GOSUB removetabs
			LET code$(icount) = LTRIM$(RTRIM$(code$(icount)))
			LET count = icount

		NEXT icount

		LET count = icount + 1

		CLOSE #2

	END IF

RETURN

usecmd:

	IF UCASE$(LEFT$(code$(count), 3)) = "USE" THEN

		LET incs$(include) = "include " + MID$(code$(count), 5, LEN(code$(count)) - 4)
		LET code$(count) = ""
		LET include = include + 1

	END IF

RETURN

letcmd:

	LET parameters$ = LTRIM$(RTRIM$(parameters$))
	LET letvar$ = LEFT$(parameters$, INSTR(parameters$, "=") - 1)
	LET infix$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(parameters$, "=") + 1, LEN(parameters$))))

	GOSUB evaluate

	IF LEFT$(letvar$, 1) = "@" THEN

		LET array$ = letvar$

		GOSUB parsearray
		GOSUB pointarray

		PRINT #1, "pop ax"
		PRINT #1, "mov [si],ax"

	ELSE

		PRINT #1, "pop ax"
		PRINT #1, "mov " + letvar$ + ",ax"

	END IF

RETURN

simplelet:

	LET operator$ = ""

	IF INSTR(parameters$, "+") THEN

		LET operator$ = "+"

	ELSEIF INSTR(parameters$, "-") THEN

		LET operator$ = "-"

	ELSEIF INSTR(parameters$, "*") THEN

		LET operator$ = "*"

	ELSEIF INSTR(parameters$, "/") THEN

		LET operator$ = "/"

	ELSE

		LET operator$ = ""

	END IF

	LET letvar$ = LTRIM$(RTRIM$(LEFT$(parameters$, INSTR(parameters$, "=") - 1)))

	IF operator$ = "" THEN

		LET operand1$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(parameters$, "=") + 1, LEN(parameters$))))
		LET operand2$ = ""

	ELSE

		LET operand1$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(parameters$, "=") + 1, INSTR(parameters$, operator$) - INSTR(parameters$, "=") - 1)))
		LET operand2$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(parameters$, operator$) + 1, LEN(parameters$))))

	END IF

	SELECT CASE operator$

		CASE ""

			PRINT #1, "mov ax," + operand1$
			PRINT #1, "mov " + letvar$ + ",ax"

		CASE "+"

			PRINT #1, "mov ax," + operand1$
			PRINT #1, "add ax," + operand2$
			PRINT #1, "mov " + letvar$ + ",ax"

		CASE "-"

			PRINT #1, "mov ax," + operand1$
			PRINT #1, "sub ax," + operand2$
			PRINT #1, "mov " + letvar$ + ",ax"

		CASE "*"

			PRINT #1, "mov ax," + operand1$
			PRINT #1, "mov bx," + operand2$
			PRINT #1, "mul bx"
			PRINT #1, "mov " + letvar$ + ",ax"

		CASE "/"

			PRINT #1, "mov dx,0"
			PRINT #1, "mov ax," + operand1$
			PRINT #1, "mov bx," + operand2$
			PRINT #1, "div bx"
			PRINT #1, "mov " + letvar$ + ",ax"

	END SELECT

RETURN

ifcmd:

	parameters$=LTRIM$(RTRIM$(parameters$))
	infix$=LEFT$(parameters$,INSTR(UCASE$(parameters$),"THEN")-1)
	jumplabel$=LTRIM$(MID$(parameters$,INSTR(UCASE$(parameters$),"THEN")+4,LEN(parameters$)))

	GOSUB evaluate

	PRINT #1, "pop ax"
	PRINT #1, "cmp ax,1"

	IF jumplabel$ = "" THEN

		LET endifcount = endifcount + 1
		LET jumplabel$ = "endif" + LTRIM$(STR$(endifcount))
		LET ifstack(ifpointer) = endifcount
		LET ifpointer = ifpointer + 1

		PRINT #1, "jne ." + jumplabel$

	ELSE

		IF LEFT$(jumplabel$,1) <> "@" THEN

			PRINT #1, "je ." + jumplabel$

		ELSE

			PRINT #1, "je " + jumplabel$

		END IF

	END IF

RETURN

simpleif:

	LET operator$ = ""

	IF INSTR(parameters$, "<>") THEN

		LET operator$ = "<>"

	ELSEIF INSTR(parameters$, "<=") THEN

		LET operator$ = "<="

	ELSEIF INSTR(parameters$, ">=") THEN

		LET operator$ = ">="

	ELSEIF INSTR(parameters$, "=") THEN

		LET operator$ = "="

	ELSEIF INSTR(parameters$, "<") THEN

		LET operator$ = "<"

	ELSEIF INSTR(parameters$, ">") THEN

		LET operator$ = ">"

	END IF

	LET operatorlength = LEN(operator$)

	LET operand1$ = LTRIM$(RTRIM$(LEFT$(parameters$, INSTR(parameters$, operator$) - 1)))
	LET operand2$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(parameters$, operator$) + operatorlength, INSTR(UCASE$(parameters$), " THEN") - INSTR(parameters$, operator$) - operatorlength)))
	LET jumplabel$ = LTRIM$(RTRIM$(MID$(parameters$, INSTR(UCASE$(parameters$), "THEN") + 5, LEN(parameters$))))

	PRINT #1, "mov ax," + operand1$
	PRINT #1, "mov bx," + operand2$
	PRINT #1, "cmp ax,bx"

	IF jumplabel$ = "" THEN

		LET endifcount = endifcount + 1
		LET jumplabel$ = "endif" + LTRIM$(STR$(endifcount))
		LET ifstack(ifpointer) = endifcount
		LET ifpointer = ifpointer + 1

		SELECT CASE operator$

			CASE "="

				PRINT #1, "jne ." + jumplabel$

			CASE "<"

				PRINT #1, "ja ." + jumplabel$

			CASE ">"

				PRINT #1, "jb ." + jumplabel$

			CASE "<>"

				PRINT #1, "je ." + jumplabel$

			CASE "<="

				PRINT #1, "ja ." + jumplabel$

			CASE ">="

				PRINT #1, "jb ." + jumplabel$

		END SELECT

	ELSE

		IF LEFT$(jumplabel$,1) = "@" THEN

			LET period$ = ""

		ELSE

			LET period$ = "."

		END IF

		SELECT CASE operator$

			CASE "="

				PRINT #1, "je " + period$ + jumplabel$

			CASE "<"

				PRINT #1, "jb " + period$ + jumplabel$

			CASE ">"

				PRINT #1, "ja " + period$ + jumplabel$

			CASE "<>"

				PRINT #1, "jne " + period$ + jumplabel$

			CASE "<="

				PRINT #1, "jbe " + period$ + jumplabel$

			CASE ">="

				PRINT #1, "jae " + period$ + jumplabel$

		END SELECT

	END IF

RETURN

echocmd:

	LET echostringflag = 1
	LET newlineflag = 1

	IF LEFT$(parameters$, 1) = CHR$(34) THEN

		IF RIGHT$(parameters$, 1) = "\" THEN

			LET nobreak = 1
			LET strechoflag = 1
			LET parameters$ = RTRIM$(LEFT$(parameters$, LEN(parameters$) - 1))

		ELSE

			LET nobreak = 0

		END IF

		LET vardata$ = MID$(parameters$, 2, LEN(parameters$) - 2)

		LET echocount = echocount + 1
		LET varname$ = "echodata" + LTRIM$(STR$(echocount))

		IF nobreak = 1 THEN

			PRINT #3, varname$ + " db '" + vardata$ + "',0"
			PRINT #1, "mov dx," + varname$
			PRINT #1, "call strecho"

		ELSE

			PRINT #3, varname$ + " db '" + vardata$ + "',13,10,36"
			PRINT #1, "mov dx," + varname$
			PRINT #1, "call echostring"

		END IF

	ELSEIF LEFT$(parameters$, 1) = "@" THEN

		LET echovalueflag = 1

		LET array$ = parameters$

		GOSUB parsearray
		GOSUB pointarray

		PRINT #1, "mov ax,[si]"
		PRINT #1, "call echovalue"

	ELSEIF RIGHT$(parameters$, 1) = "$" THEN

		LET strechoflag = 1

		LET parameters$ = LEFT$(parameters$,LEN(parameters$)-1)

		PRINT #1, "mov dx," + parameters$
		PRINT #1, "call strecho"

	ELSE

		LET echovalueflag = 1

		PRINT #1, "mov ax," + parameters$
		PRINT #1, "call echovalue"

	END IF

RETURN

inputcmd:

	IF RIGHT$(parameters$, 1) = "$" THEN

		LET strinputflag = 1

		LET parameters$ = LEFT$(parameters$,LEN(parameters$)-1)

		PRINT #1,"mov di," + parameters$
		PRINT #1,"mov dx,1024"
		PRINT #1,"call strinput"

	ELSE

		LET inputflag = 1

		PRINT #1, "call inputvalue"

		IF LEFT$(parameters$, 1) = "@" THEN

			LET array$ = parameters$

			PRINT #1, "push ax"

			GOSUB parsearray
			GOSUB pointarray

			PRINT #1, "pop ax"
			PRINT #1, "mov [si],ax"


		ELSE

			PRINT #1, "mov " + parameters$ + ",ax"

		END IF

	END IF

RETURN

copycmd:

	LET comma = INSTR(parameters$, ",")
	LET string1$ = LEFT$(parameters$, comma - 1)
	LET string2$ = MID$(parameters$, comma + 1, LEN(parameters$) - comma)
	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	IF LEFT$(string2$,1) = CHR$(34) THEN

		LET string2$ = MID$(string2$, 2, LEN(string2$) - 2)
		LET copystring = copystring + 1
		LET name$ = "copystr" + LTRIM$(STR$(copystring))

		PRINT #3, name$ + " db '" + string2$ + "',0"
		PRINT #3, "rb 1023"

		LET string2$ = name$

	ELSE

		LET string2$ = LEFT$(string2$,LEN(string2$)-1)

	END IF

	PRINT #1,"mov di," + string1$
	PRINT #1,"mov si," + string2$
	PRINT #1,"mov dx,1024"
	PRINT #1,"call strcopy"

RETURN

concatcmd:

	LET comma = INSTR(parameters$, ",")
	LET string1$ = LEFT$(parameters$, comma - 1)
	LET string2$ = MID$(parameters$, comma + 1, LEN(parameters$) - comma)
	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	IF LEFT$(string2$,1) = CHR$(34) THEN

		LET string2$ = MID$(string2$, 2, LEN(string2$) - 2)
		LET concatstring = concatstring + 1
		LET name$ = "concatstr" + LTRIM$(STR$(concatstring))

		PRINT #3, name$ + " db '" + string2$ + "',0"
		PRINT #3, "rb 1023"

		LET string2$ = name$

	ELSE

		LET string2$ = LEFT$(string2$,LEN(string2$)-1)

	END IF

	PRINT #1,"mov si," + string2$
	PRINT #1,"mov di," + string1$
	PRINT #1,"mov dx,1024"
	PRINT #1,"call strconcat"

RETURN

comparecmd:

	LET comma = INSTR(parameters$, ",")
	LET retvar$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET comma = INSTR(parameters$, ",")
	LET string1$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET string2$ = parameters$

	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	IF LEFT$(string2$,1) = CHR$(34) THEN

		LET string2$ = MID$(string2$, 2, LEN(string2$) - 2)
		LET comparestring = comparestring + 1
		LET name$ = "cmpstr" + LTRIM$(STR$(comparestring))

		PRINT #3, name$ + " db '" + string2$ + "',0"
		PRINT #3, "rb 1023"

		LET string2$ = name$

	ELSE

		LET string2$ = LEFT$(string2$,LEN(string2$)-1)

	END IF

	PRINT #1, "mov ax," + string1$
	PRINT #1, "mov bx," + string2$
	PRINT #1, "call strcmp"
	PRINT #1, "mov " + retvar$ + ",cx"

RETURN

lengthcmd:

	LET comma = INSTR(parameters$, ",")
	LET retvar$ = LEFT$(parameters$, comma - 1)
	LET string1$ = MID$(parameters$, comma + 1, LEN(parameters$) - comma)
	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	PRINT #1, "mov ax," + string1$
	PRINT #1, "call strlen"
	PRINT #1, "mov " + retvar$ + ",cx"

RETURN

findcmd:

	LET comma = INSTR(parameters$, ",")
	LET retvar$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET comma = INSTR(parameters$, ",")
	LET string1$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET string2$ = parameters$

	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	IF RIGHT$(string2$,1) = "$" THEN

		LET string2$ = LEFT$(string2$,LEN(string2$)-1)

	END IF

	PRINT #1, "mov ax," + string1$
	PRINT #1, "mov bl," + string2$
	PRINT #1, "call strfind"
	PRINT #1, "mov si," + string1$
	PRINT #1, "sub cx,si"
	PRINT #1, "mov ax,cx"
	PRINT #1, "add ax,1"
	PRINT #1, "cmp ax,1024"
	PRINT #1, "jb .isfound"
	PRINT #1, "xor ax,ax"
	PRINT #1, ".isfound:"
	PRINT #1, "mov " + retvar$ + ",ax"

RETURN

trimcmd:

	LET parameters$ = LEFT$(parameters$,LEN(parameters$)-1)

	PRINT #1, "mov ax," + parameters$
	PRINT #1, "call strtrim"

RETURN

cutcmd:

	LET comma = INSTR(parameters$, ",")
	LET strstart$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET comma = INSTR(parameters$, ",")
	LET strcount$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET comma = INSTR(parameters$, ",")
	LET string1$ = LEFT$(parameters$, comma - 1)

	LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
	LET string2$ = parameters$

	LET string1$ = LEFT$(string1$,LEN(string1$)-1)
	LET string2$ = LEFT$(string2$,LEN(string2$)-1)

	PRINT #1, "mov cx," + strcount$
	PRINT #1, "mov si," + string2$
	PRINT #1, "add si," + strstart$
	PRINT #1, "mov di," + string1$
	PRINT #1, "call strcut"

RETURN

intstrcmd:

	LET comma = INSTR(parameters$, ",")
	LET string1$ = LEFT$(parameters$, comma - 1)
	LET operand1$ = MID$(parameters$, comma + 1, LEN(parameters$) - comma)
	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	PRINT #1, "mov cx,10"
	PRINT #1, "mov ax," + operand1$
	PRINT #1, "lea si,[" + string1$ + "]"
	PRINT #1, "call strintstr"

RETURN

strintcmd:

	LET comma = INSTR(parameters$, ",")
	LET operand1$ = LEFT$(parameters$, comma - 1)
	LET string1$ = MID$(parameters$, comma + 1, LEN(parameters$) - comma)
	LET string1$ = LEFT$(string1$,LEN(string1$)-1)

	PRINT #1, "mov cx,10"
	PRINT #1, "lea si,[" + string1$ + "]"
	PRINT #1, "call strstrint"
	PRINT #1, "mov " + operand1$ + ",ax"


RETURN

uppercmd:

	LET parameters$ = LEFT$(parameters$,LEN(parameters$)-1)

	PRINT #1,"mov di," + parameters$
	PRINT #1,"mov dx,1024"
	PRINT #1,"call strupper"

RETURN

lowercmd:

	LET parameters$ = LEFT$(parameters$,LEN(parameters$)-1)

	PRINT #1,"mov di," + parameters$
	PRINT #1,"mov dx,1024"
	PRINT #1,"call strlower"

RETURN

clscmd:

	IF parameters$ = "" THEN

		PRINT #1, "call resetcolors"

	ELSE

		LET comma = INSTR(parameters$, ",")
		LET clscolor$ = LEFT$(parameters$, comma - 1)

		LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
		LET comma = INSTR(parameters$, ",")
		LET scroll$ = LEFT$(parameters$, comma - 1)

		LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
		LET comma = INSTR(parameters$, ",")
		LET colorx1$ = LEFT$(parameters$, comma - 1)

		LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
		LET comma = INSTR(parameters$, ",")
		LET colorx2$ = LEFT$(parameters$, comma - 1)

		LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
		LET comma = INSTR(parameters$, ",")
		LET colory1$ = LEFT$(parameters$, comma - 1)

		LET parameters$ = MID$(parameters$, comma + 1, LEN(parameters$) - 1)
		LET colory2$ = parameters$

		IF LEFT$(clscolor$, 1) = "[" THEN

			PRINT #1, "mov bh,byte " + clscolor$

		ELSE

			PRINT #1, "mov bh," + clscolor$

		END IF
 
		IF LEFT$(scroll$, 1) = "[" THEN

			PRINT #1, "mov al,byte " + scroll$

		ELSE

			PRINT #1, "mov al," + scroll$

		END IF

		IF LEFT$(colorx1$, 1) = "[" THEN

			PRINT #1, "mov cl,byte " + colorx1$

		ELSE

			PRINT #1, "mov cl," + colorx1$

		END IF

		IF LEFT$(colorx2$, 1) = "[" THEN

			PRINT #1, "mov dl,byte " + colorx2$

		ELSE

			PRINT #1, "mov dl," + colorx2$

		END IF

		IF LEFT$(colory1$, 1) = "[" THEN

			PRINT #1, "mov ch,byte " + colory1$

		ELSE

			PRINT #1, "mov ch," + colory1$

		END IF

		IF LEFT$(colory2$, 1) = "[" THEN

			PRINT #1, "mov dh,byte " + colory2$

		ELSE

			PRINT #1, "mov dh," + colory2$

		END IF

	END IF

	PRINT #1, "call clearscreen"

RETURN

cursorcmd:

	LET comma = INSTR(parameters$, ",")
	LET cursorx$ = LEFT$(parameters$, comma - 1)
	LET cursory$ = MID$(parameters$, comma + 1, LEN(parameters$) - comma)

	IF LEFT$(cursory$, 1) = "[" THEN

		PRINT #1, "mov dh,byte " + cursory$

	ELSE

		PRINT #1, "mov dh," + cursory$

	END IF

	IF LEFT$(cursorx$, 1) = "[" THEN

		PRINT #1, "mov dl,byte " + cursorx$

	ELSE

		PRINT #1, "mov dl," + cursorx$

	END IF

	PRINT #1, "call movecursor"

RETURN

chrcmd:

	IF LEFT$(parameters$, 1) = "[" THEN

		PRINT #1, "mov dl,byte " + parameters$

	ELSE

		PRINT #1, "mov dl," + parameters$

	END IF
				
	PRINT #1, "call echochar"

RETURN

waitcmd:

	LET parameters$ = HEX$(VAL(parameters$))
	LET chars = LEN(parameters$)

	IF chars > 4 THEN

	        LET regcx$ = LEFT$(parameters$, chars - 4)
	        LET regdx$ = RIGHT$(parameters$, 4)

	ELSE

	        LET regdx$ = parameters$

	END IF

	PRINT #1, "mov cx,0" + regcx$ + "h"
	PRINT #1, "mov dx,0" + regdx$ + "h"
	PRINT #1, "call waitfor"

RETURN

tokenize:

	LET infix$="("+infix$+")"

	FOR replacetoken=1 TO LEN(infix$)

		IF INSTR(infix$,"<=") THEN

			LET leftfix$=LEFT$(infix$,INSTR(infix$,"<=")-1)
			LET infix$=RIGHT$(infix$,LEN(infix$)-LEN(leftfix$)-2)
			LET infix$=leftfix$+CHR$(128)+infix$
   
		ENDIF

		IF INSTR(infix$,">=") THEN

			LET leftfix$=LEFT$(infix$,INSTR(infix$,">=")-1)
			LET infix$=RIGHT$(infix$,LEN(infix$)-LEN(leftfix$)-2)
			LET infix$=leftfix$+CHR$(129)+infix$
   
		ENDIF

		IF INSTR(infix$,"<>") THEN

			LET leftfix$=LEFT$(infix$,INSTR(infix$,"<>")-1)
			LET infix$=RIGHT$(infix$,LEN(infix$)-LEN(leftfix$)-2)
			LET infix$=leftfix$+CHR$(130)+infix$
   
		ENDIF

	NEXT replacetoken

RETURN

translate:

	LET postfix$ = ""
	LET operatorstack$ = ""
	LET token$ = ""
	LET operator$ = ""

	FOR tokenpos = 1 TO LEN(infix$)

		LET token$ = MID$(infix$, tokenpos, 1)

		SELECT CASE token$

			CASE "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"

				LET postfix$ = postfix$ + token$

			CASE "[","@"

				GOSUB parsevariable

			CASE "("

				LET operatorstack$ = operatorstack$ + token$

			CASE ")"

				WHILE RIGHT$(operatorstack$, 1) <> "("

					LET postfix$ = postfix$ + " " + RIGHT$(operatorstack$, 1)
					LET operatorstack$ = LEFT$(operatorstack$, LEN(operatorstack$) - 1)

				WEND

				LET operatorstack$ = LEFT$(operatorstack$, LEN(operatorstack$) - 1)

			CASE "*", "/", "+", "-", "<", ">", CHR$(128), CHR$(129), "=", CHR$(130), "&", "|"

				GOSUB processoperators

		END SELECT

	NEXT

	LET newstack$ = ""

	FOR tokenpos = 1 TO LEN(operatorstack$)

		LET newstack$ = newstack$ + " " + MID$(operatorstack$, tokenpos, 1)

	NEXT

	LET operatorstack$ = newstack$

	LET postfix$ = postfix$ + operatorstack$

RETURN

processoperators:

	GOSUB getprecedence

	WHILE operatorstack$<>"" AND RIGHT$(operatorstack$, 1)<>"(" AND tokenprecedence<=stackprecedence

		LET postfix$=postfix$+" "+RIGHT$(operatorstack$,1)
		LET operatorstack$=LEFT$(operatorstack$,LEN(operatorstack$)-1)

		GOSUB getprecedence

	WEND

	LET operatorstack$=operatorstack$+token$
	LET postfix$=postfix$+" "

RETURN

evaluate:

	GOSUB tokenize
	GOSUB translate

	WHILE LEN(postfix$)

		IF INSTR(postfix$, " ") THEN

			LET token$ = LEFT$(postfix$, INSTR(postfix$, " ") - 1)
			LET postfix$ = RIGHT$(postfix$, LEN(postfix$) - LEN(token$) - 1)

		ELSE

			LET token$ = postfix$
			LET postfix$ = ""

		END IF

		SELECT CASE LEFT$(token$, 1)

			CASE "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "["

				PRINT #1, "push " + token$

			CASE "@"

				LET array$ = token$

				GOSUB parsearray
				GOSUB pointarray

				PRINT #1, "mov ax,[si]"
				PRINT #1, "push ax"

			CASE "*", "/", "+", "-", "<", ">", CHR$(128), CHR$(129), "=", CHR$(130), "&", "|"

				LET operator$ = token$
				GOSUB calculate

		END SELECT

	WEND

RETURN

calculate:

	PRINT #1, "pop [operand2]"
	PRINT #1, "pop [operand1]"

	SELECT CASE operator$

		CASE "*"

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "mov bx,[operand2]"
			PRINT #1, "mul bx"

		CASE "/"

			PRINT #1, "mov dx,0"
			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "mov bx,[operand2]"
			PRINT #1, "div bx"

		CASE "+"

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "add ax,[operand2]"

		CASE "-"

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "sub ax,[operand2]"


		CASE "<"

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "cmp ax,[operand2]"
			PRINT #1, "setb al"
			PRINT #1, "and ax,255"

		CASE ">"

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "cmp ax,[operand2]"
			PRINT #1, "seta al"
			PRINT #1, "and ax,255"

		CASE CHR$(128) '<=

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "cmp ax,[operand2]"
			PRINT #1, "setbe al"
			PRINT #1, "and ax,255"

		CASE CHR$(129) '>=

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "cmp ax,[operand2]"
			PRINT #1, "setae al"
			PRINT #1, "and ax,255"

		CASE "="

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "cmp ax,[operand2]"
			PRINT #1, "setz al"
			PRINT #1, "and ax,255"

		CASE CHR$(130) '<>

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "cmp ax,[operand2]"
			PRINT #1, "setnz al"
			PRINT #1, "and ax,255"

		CASE "&" 'and

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "and ax,[operand2]"
			
		CASE "|" 'or

			PRINT #1, "mov ax,[operand1]"
			PRINT #1, "or ax,[operand2]"

	END SELECT

	PRINT #1, "push ax"

RETURN

getprecedence:

	LET stacktop$ = RIGHT$(operatorstack$, 1)

	IF stacktop$ = "|" THEN

		LET stackprecedence = 1

	ELSEIF stacktop$ = "&" THEN

		LET stackprecedence = 2

	ELSEIF stacktop$ = "=" OR stacktop$ = CHR$(130) THEN

		LET stackprecedence = 3

	ELSEIF stacktop$ = "<" OR stacktop$ = ">" OR stacktop$ = CHR$(128) OR stacktop$ = CHR$(129) THEN

		LET stackprecedence = 4

	ELSEIF stacktop$ = "+" OR stacktop$ = "-" THEN

		LET stackprecedence = 5

	ELSEIF stacktop$ = "*" OR stacktop$ = "/" THEN

		LET stackprecedence = 6

	ELSE

		LET stackprecedence = 0

	END IF

	IF token$ = "|" THEN

		LET tokenprecedence = 1

	ELSEIF token$ = "&" THEN

		LET tokenprecedence = 2

	ELSEIF token$ = "=" OR token$ = CHR$(130) THEN

		LET tokenprecedence = 3	


	ELSEIF token$ = "<" OR token$ = ">" OR token$ = CHR$(128) OR token$ = CHR$(129) THEN

		LET tokenprecedence = 4

	ELSEIF token$ = "+" OR token$ = "-" THEN

		LET tokenprecedence = 5

	ELSEIF token$ = "*" OR token$ = "/" THEN

		LET tokenprecedence = 6

	ELSE

		LET tokenprecedence = 0

	END IF

RETURN

parsevariable:

	LET var$ = ""

	IF token$="@" THEN

		LET arrayflag = 1

	ELSE

		LET arrayflag = 0

	END IF

	FOR varpos = tokenpos TO LEN(infix$)

		LET token$ = MID$(infix$, varpos, 1)

		SELECT CASE token$

			CASE "]"

				LET var$ = var$ + token$

				IF arrayflag=0 THEN

					EXIT FOR

				END IF

			CASE "}"

				LET var$ = var$ + token$
				EXIT FOR

			CASE ELSE

				LET var$ = var$ + token$

		END SELECT

	NEXT varpos

	LET tokenpos = varpos - 1

	LET postfix$ = postfix$ + LTRIM$(RTRIM$(var$))

RETURN

parsearray:

	LET array$ = RIGHT$(array$, LEN(array$) - 1)
	LET offset$ = RTRIM$(RIGHT$(array$, (LEN(array$) - INSTR(array$, "{"))))
	LET offset$ = LEFT$(offset$, LEN(offset$) - 1)
	LET array$ = LEFT$(array$, INSTR(array$, "{") - 1)

RETURN

pointarray:

	PRINT #1, "mov ax," + offset$
	PRINT #1, "mov bx,2"
	PRINT #1, "mul bx"
	PRINT #1, "mov si," + array$
	PRINT #1, "add si,ax"

RETURN